Package: prefio
Title: Structures for Preference Data
Description: Convenient structures for creating, sourcing, reading, writing
  and manipulating ordinal preference data. Methods for writing to/from PrefLib
  formats. See Nicholas Mattei and Toby Walsh "PrefLib: A Library of Preference
  Data" (2013) <doi:10.1007/978-3-642-41575-3_20>.
Authors@R: c(
  person("Floyd", "Everest", email = "me@floydeverest.com",
         role = c("aut", "cre"), comment = c(ORCID = "0000-0002-2726-6736")),
  person("Heather", "Turner", email = "ht@heatherturner.net",
         role = c("aut"), comment = c(ORCID = "0000-0002-1256-3375")),
  person("Damjan", "Vukcevic", email = "damjan@vukcevic.net",
         role = c("aut"), comment = c(ORCID = "0000-0001-7780-9586")))
Version: 0.1.1
Depends: R (>= 2.10)
LazyData: true
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
URL: https://github.com/fleverest/prefio/,
        https://fleverest.github.io/prefio/
BugReports: https://github.com/fleverest/prefio/issues/
Imports: dplyr, magrittr, tidyr, stats
Suggests: covr, testthat (>= 3.0.0)
Config/testthat/edition: 3
Author: Floyd Everest [aut, cre] (<https://orcid.org/0000-0002-2726-6736>),
  Heather Turner [aut] (<https://orcid.org/0000-0002-1256-3375>),
  Damjan Vukcevic [aut] (<https://orcid.org/0000-0001-7780-9586>)
Maintainer: Floyd Everest <me@floydeverest.com>
Built: R 4.4.0; ; 2024-04-30 20:05:26 UTC; unix
