# Generated by roxygen2: do not edit by hand

S3method("[",aggregated_preferences)
S3method("[",grouped_preferences)
S3method("[",preferences)
S3method("names<-",preferences)
S3method(Ops,preferences)
S3method(aggregate,aggregated_preferences)
S3method(aggregate,preferences)
S3method(as.aggregated_preferences,aggregated_preferences)
S3method(as.aggregated_preferences,preferences)
S3method(as.data.frame,grouped_preferences)
S3method(as.data.frame,preferences)
S3method(as.matrix,aggregated_preferences)
S3method(as.matrix,preferences)
S3method(as.preferences,aggregated_preferences)
S3method(as.preferences,default)
S3method(as.preferences,grouped_preferences)
S3method(as.preferences,matrix)
S3method(format,grouped_preferences)
S3method(format,preferences)
S3method(group,preferences)
S3method(is.na,grouped_preferences)
S3method(is.na,preferences)
S3method(length,preferences)
S3method(names,preferences)
S3method(print,choices)
S3method(print,grouped_preferences)
S3method(print,preferences)
S3method(rbind,aggregated_preferences)
S3method(rbind,preferences)
S3method(rep,preferences)
export("%>%")
export(adjacency)
export(as.aggregated_preferences)
export(as.preferences)
export(choices)
export(frequencies)
export(group)
export(preferences)
export(read_preflib)
export(str)
export(write_preflib)
importFrom(magrittr,"%>%")
importFrom(stats,aggregate)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(utils,read.csv)
importFrom(utils,str)
